#!/usr/bin/env bash

# ********** Please don't edit this file!
# ********** It has been generated automatically by dae_devops version 0.5.2.
# ********** For repository_name soakdb3

me=${BASH_SOURCE}
echo "${me}: installing 0 dependencies for repository_name soakdb3"

function __install {
    echo ${me}: "$@"
    "$@"
}


# dae_devops_fingerprint af7f370578b9e81ef22fb808092418e9