class Keywords:
    COMMAND = "soakdb3::datafaces::keywords::command"
    PAYLOAD = "soakdb3::datafaces::keywords::payload"


class Commands:
    EXECUTE = "soakdb3::datafaces::commands::execute"


class DatafaceTypes:
    AIOHTTP = "soakdb3_lib.datafaces.aiohttp"
    DIRECT = "soakdb3_lib.datafaces.aiosqlite"
