from dls_servbase_api.aiohttp_client import AiohttpClient as DlsServbaseAiohttpClient


# ------------------------------------------------------------------------------------------
class AiohttpClient(DlsServbaseAiohttpClient):
    """
    Object representing a client which makes aiohttp requests.
    """

    pass
